@@ -19,7 +19,7 @@
 /*
  * Server packet routines to send packets. 
  */
-/* $Id: packet_send.c,v 1.115 2002/10/05 18:46:19 priikone Exp $ */
+/* $Id: packet_send.c,v 1.119 2002/10/24 12:19:00 priikone Exp $ */
 
 #include "serverincludes.h"
 #include "server_internal.h"
@@ -168,7 +168,8 @@ void silc_server_packet_send_dest(SilcServer server,
     return;
   }
 
-  SILC_LOG_DEBUG(("Sending %s packet", silc_get_packet_name(type)));
+  SILC_LOG_DEBUG(("Sending %s packet (forced=%s)",
+		  silc_get_packet_name(type), force_send ? "yes" : "no"));
 
   if (dst_id) {
     dst_id_data = silc_id_id2str(dst_id, dst_id_type);
@@ -882,6 +883,20 @@ void silc_server_packet_relay_to_channel(SilcServer server,
 	  continue;
 	gone = TRUE;
 
+	/* If we are backup router and remote is our primary router and
+	   we are currently doing backup resuming protocol we must not
+	   re-encrypt message with session key. */
+	if (server->backup_router && SILC_SERVER_IS_BACKUP(sock) &&
+	    SILC_PRIMARY_ROUTE(server) == sock) {
+	  silc_server_packet_send_to_channel_real(server, sock, &packetdata,
+						  idata->send_key,
+						  idata->hmac_send,
+						  idata->psn_send++,
+						  data, data_len, TRUE,
+						  force_send);
+	  continue;
+	}
+
 	SILC_LOG_DEBUG(("Remote is router, encrypt with session key"));
 
 	/* If private key mode is not set then decrypt the packet
@@ -1048,6 +1063,7 @@ void silc_server_send_motd(SilcServer server,
     if (!motd)
       return;
 
+    motd[motd_len] = 0;
     silc_server_send_notify(server, sock, FALSE, SILC_NOTIFY_TYPE_MOTD, 1,
 			    motd, motd_len);
     silc_free(motd);
@@ -1066,7 +1082,7 @@ void silc_server_send_error(SilcServer server,
 
   memset(buf, 0, sizeof(buf));
   va_start(ap, fmt);
-  vsprintf(buf, fmt, ap);
+  vsnprintf(buf, sizeof(buf) - 1, fmt, ap);
   va_end(ap);
 
   silc_server_packet_send(server, sock, SILC_PACKET_ERROR, 0, 
@@ -1764,6 +1780,7 @@ void silc_server_send_channel_key(SilcServer server,
   SilcBuffer packet;
   unsigned char *chid;
   SilcUInt32 tmp_len;
+  const char *cipher;
  
   SILC_LOG_DEBUG(("Sending key to channel %s", channel->channel_name));
  
@@ -1775,11 +1792,11 @@ void silc_server_send_channel_key(SilcServer server,
     return;
  
   /* Encode channel key packet */
-  tmp_len = strlen(channel->channel_key->cipher->name);
+  cipher = silc_cipher_get_name(channel->channel_key);
+  tmp_len = strlen(cipher);
   packet = silc_channel_key_payload_encode(silc_id_get_len(channel->id,
 							   SILC_ID_CHANNEL),
-					   chid, tmp_len,
-                                           channel->channel_key->cipher->name,
+					   chid, tmp_len, cipher,
                                            channel->key_len / 8, channel->key);
   silc_server_packet_send_to_channel(server, sender, channel, 
 				     SILC_PACKET_CHANNEL_KEY,
